import z3
from miasm2.analysis.machine import Machine
from miasm2.jitter.csts import PAGE_READ, PAGE_WRITE
from miasm2.analysis.dse import DSEPathConstraint
from miasm2.core.interval import interval
from miasm2.expression.expression import *

def force_alphanum(x):
    """ Force x in [A-Za-z0-9] """
    constraint = []
    # Integer
    constraint.append(z3.And(x <= ord('9'), x >= ord('0')))

    # Uppercase
    constraint.append(z3.And(x <= ord('Z'), x >= ord('A')))

    # Lowercase
    constraint.append(z3.And(x <= ord('z'), x >= ord('a')))

    return z3.Or(*constraint)


# Init and map the shellcode
data = open("shellcode.bin").read()
machine = Machine("x86_32")

jitter = machine.jitter()
addr_sc = 0x400000
jitter.vm.add_memory_page(addr_sc, PAGE_READ|PAGE_WRITE, data, "Shellcode")

# Init environment: stack, and EAX on the entry_point
jitter.init_stack()
jitter.cpu.EAX = addr_sc

# Init the dse
dse = DSEPathConstraint(machine, produce_solution=False)

# Add a breakpoint before the OEP
def jump_on_oep(jitter):
    print "OEP reached!"
    return False
jitter.add_breakpoint(addr_sc + 0x4b, jump_on_oep)

# Init jitter context and DSE
jitter.init_run(addr_sc)
dse.attach(jitter)
dse.update_state_from_concrete()
dse.symbolize_memory(interval([(addr_sc, addr_sc + len(data) - 1)]))

# Launch the jitter
jitter.continue_run()


# Force final state
memory = jitter.vm.get_mem(addr_sc, 0x368)
# > url == garbage
urls = ["https://miasm.re/1", "https://github.com/cea-sec/miasm"]
memory += "\x00".join(urls) + "\x00\x00"

for i, c in enumerate(memory):
    addr = addr_sc + i
    exp = ExprMem(ExprInt(addr, 32), 8)
    value = ExprInt(ord(c), 8)
    eq = dse.eval_expr(exp)
    # Constraint the result of the equation at address "addr" (in "eq") to be
    # equal to our custom memory byte "value"
    eaff = ExprAff(eq, value)
    dse.cur_solver.add(dse.z3_trans.from_expr(eaff))


# Force alphanum
for addr in xrange(addr_sc, addr_sc + len(data)):
    z3_addr = dse.z3_trans.from_expr(dse.memory_to_expr(addr))
    dse.cur_solver.add(force_alphanum(z3_addr))

# Get the solution
dse.cur_solver.check()
model = dse.cur_solver.model()
out = ""
for addr in xrange(addr_sc, addr_sc + len(data)):
    x = model.eval(dse.z3_trans.from_expr(dse.memory_to_expr(addr)))
    try:
        out += chr(x.as_long())
    except:
        # No constraint on this input, let's use an arbitrary value
        out += "A"

# Out the crafted shellcode
print out
